
"""
Time of the day

Initialize: This class definition is a different way
to store time as simply seconds. It requires conversions
but simplifies internal processing of different functions.

"""

class Time(object):
    def __init__(self, hour=0, mins=0, sec=0):
        """Military time: 24 hr format """
        hour, mins, sec = self.normalize(hour, mins, sec)
        self.sec = sec + mins*60 + hour*60*60
    
    def normalize(self, hour, mins, sec):
        """This function converts hours/mins/seconds to seconds,
        getting rid of any extra value. This function can also 
        be converted to a static function since it does not 
        actually use any of the internally stored data .
        
        """
        mins += sec//60
        sec = sec%60
        hour += mins//60
        mins = mins%60
        hour = hour%24
        return (hour, mins, sec)
    
    def convert(self, seconds):
        """This function converts seconds to hour, minute 
        and seconds. This function can also be converted to
        a static function since it does not actually use 
        any of the internally stored data .
        
        """
        
        hour = seconds//3600
        mins = (seconds%3600)//60
        sec = (seconds%3600)%60
        return (hour, mins, sec)
    
    def __str__(self):
        hour, mins, sec = self.convert(self.sec)
        return "{}:{}:{}".format(hour, mins, sec)
        
    def __add__(self, other):
        """Add two times and return a new time value. Normalization
        occurs when we create a new time value. 
        
        """
        
        hour, mins, sec = self.convert(self.sec + other.sec)
        return Time(hour, mins, sec)
    
    def advance_time(self, hour_add, mins_add, sec_add):
        """Change the current time by adding more values""" 
        self.sec += sec_add + mins_add*60 + hour_add*60*60
        self.sec = self.sec % (24*3600) ##normalize
        
    
    def __sub__(self, other):
        new_seconds = self.sec - other.sec
        if new_seconds < 0:
            new_seconds += 24*3600
        hour, mins, sec = self.convert(new_seconds)
        return Time( hour, mins, sec)
    
    def __eq__(self, other):
        return self.sec == other.sec
 
    def __gt__(self, other):
        return self.sec > other.sec
    
if __name__ == "__main__":
    t1 = Time(23, 54, 34)
    t1.advance_time(12,34,56)
    print (t1)
    
    t2 = Time(12, 34, 56)
    ##advance time does not return anythinh
    ## but changes t1
    
    t3 = t1 + t2
    print(t3)
    
    ##creates and returns a new object t3